@@ -1,4 +1,4 @@
-(* $Id: pxp_aux.ml,v 1.18 2003/11/04 22:45:07 gerd Exp $
+(* $Id: pxp_aux.ml 690 2004-08-08 20:22:54Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -23,14 +23,6 @@ end;;
 
 module Str_hashtbl = Hashtbl.Make(HashedString);;
 
-module StringOrd = struct
-  type t = string
-  let compare = (compare : string -> string -> int)
-end;;
-
-module StringMap = Map.Make(StringOrd);;
-  (* 'a StringMap.t: the type of maps (dictionaries) from string to 'a *)
-
 let character ?swarner enc warner k =
   assert (k>=0);
   if (k >= 0xd800 & k < 0xe000) or (k >= 0xfffe & k <= 0xffff) or k > 0x10ffff
@@ -60,13 +52,14 @@ let check_name ?swarner warner name =
 ;;
 
 
-let tokens_of_content_string lexerset s =
+let tokens_of_content_string lfactory s =
   (* tokenizes general entities and character entities *)
-  let lexbuf = Pxp_lexing.from_string_inplace s in
+  let lexobj = lfactory # open_string_inplace s in
+  let scan   = lexobj # scan_content_string in
   let rec next_token () =
-    match lexerset.scan_content_string lexbuf with
+    match scan() with
 	Eof        -> []
-      | CharData _ -> let tok = CharData (Lexing.lexeme lexbuf) in
+      | CharData _ -> let tok = CharData lexobj#lexeme in
 	              tok :: next_token()
       | tok        -> tok :: next_token()
   in
@@ -76,14 +69,14 @@ let tokens_of_content_string lexerset s =
 
 exception Quick_exit;;
 
-let rec expand_attvalue_with_rec_check lexbuf l lexerset dtd entities norm_crlf =
+let rec expand_attvalue_with_rec_check (lexobj : lexer_obj) l dtd entities norm_crlf =
   (* recursively expands general entities and character entities;
    * checks "standalone" document declaration;
    * normalizes whitespace
    *
    * Exception: Quick_exit: the expanded value is equal to s
    *)
-  match lexerset.scan_content_string lexbuf with
+  match lexobj # scan_content_string () with
       Eof -> []
     | ERef n ->
 	if List.mem n entities then
@@ -98,55 +91,61 @@ let rec expand_attvalue_with_rec_check lexbuf l lexerset dtd entities norm_crlf
 	let l' =
 	  try
 	    expand_attvalue_with_rec_check
-	      (Pxp_lexing.from_string_inplace rtext)
+	      (lexobj # factory # open_string_inplace rtext)
 	      (String.length rtext)
-	      lexerset dtd (n :: entities) false
+	      dtd (n :: entities) false
 	  with
 	      Quick_exit -> [rtext]
 	  in
 	  l' @ expand_attvalue_with_rec_check
-	         lexbuf l lexerset dtd entities norm_crlf
+	         lexobj l dtd entities norm_crlf
     | CRef(-1) ->
 	if norm_crlf then begin
 	  " " :: expand_attvalue_with_rec_check
-	             lexbuf l lexerset dtd entities norm_crlf
+	             lexobj l dtd entities norm_crlf
 	end
 	else begin
 	  "  " :: expand_attvalue_with_rec_check
- 	              lexbuf l lexerset dtd entities norm_crlf
+ 	              lexobj l dtd entities norm_crlf
 	end
     | CRef n ->
-	(character ?swarner:dtd#swarner lexerset.lex_encoding dtd#warner n) ::
+	(character ?swarner:dtd#swarner lexobj#encoding dtd#warner n) ::
 	expand_attvalue_with_rec_check
- 	    lexbuf l lexerset dtd entities norm_crlf
+ 	    lexobj l dtd entities norm_crlf
     | CharData _  ->
-	if Lexing.lexeme_char lexbuf 0 = '<' then
-	  raise
-	    (WF_error
-	       ("Attribute value contains character '<' literally"));
-	if Pxp_lexing.lexeme_len lexbuf = l then
+	let ll = lexobj # lexeme_strlen in
+	if ll > 1 && ll = l then
 	   raise Quick_exit
 	else
-	  let x = Lexing.lexeme lexbuf in
+	  let x = lexobj # lexeme in
+	  if x.[0] = '<' then  (* or better: x = "<", ensured by the lexer *)
+	    raise
+	      (WF_error
+		 ("Attribute value contains character '<' literally"));
 	  x :: expand_attvalue_with_rec_check
- 	           lexbuf l lexerset dtd entities norm_crlf
+ 	           lexobj l dtd entities norm_crlf
     | _ -> assert false
 ;;
 
 
-let expand_attvalue lexbuf lexerset dtd s norm_crlf =
+let expand_attvalue (lexobj : lexer_obj) dtd s norm_crlf =
   (* norm_crlf: whether the sequence CRLF is recognized as one character or
    * not (i.e. two characters).
    * lexbuf: must result from a previous Lexing.from_string
    *)
+  (* print_string ("expand_attvalue \"" ^ s ^ "\" = "); *)
   try
-    Pxp_lexing.from_another_string_inplace lexbuf s;
+    lexobj # open_string_inplace s;
     let l =
       expand_attvalue_with_rec_check
-	lexbuf (String.length s) lexerset dtd [] norm_crlf in
-    String.concat "" l
+	lexobj (String.length s) dtd [] norm_crlf in
+    let s' =
+      String.concat "" l in
+    (* print_string ("\"" ^ s' ^ "\"\n"); *)
+    s'
   with
       Quick_exit ->
+	(* print_string ("\"" ^ s ^ "\"\n"); *)
 	s
 ;;
 
@@ -185,10 +184,11 @@ let count_lines lc s =
 ;;
 
 
-let tokens_of_xml_pi lexers s =
-  let lexbuf = Pxp_lexing.from_string_inplace (s ^ " ") in
+let tokens_of_xml_pi (lfactory : lexer_factory) s =
+  let lexobj = lfactory # open_string_inplace (s ^ " ") in
+  let scan = lexobj # scan_xml_pi in
   let rec collect () =
-    let t = lexers.scan_xml_pi lexbuf in
+    let t = scan() in
     match t with
 	Pro_eof -> []
       | _       -> t :: collect()
@@ -283,7 +283,7 @@ let rec count pred l =
 (**********************************************************************)
 (* attributes *)
 
-let check_attribute_value_lexically lexerset x t v =
+let check_attribute_value_lexically (lfactory:lexer_factory) x t v =
   (* raises x if the attribute value v does not match the lexical rules
    * for attribute type t:
    * - t = A_id: v must be a <name>
@@ -297,9 +297,10 @@ let check_attribute_value_lexically lexerset x t v =
    * - t = A_enum _: v must match <nmtoken>
    * - t = A_cdata: not checked
    *)
-  let lexbuf = Pxp_lexing.from_string_inplace v in
+  let lexobj = lfactory # open_string_inplace v in
+  let scan = lexobj#scan_name_string in
   let rec get_name_list() =
-    match lexerset.scan_name_string lexbuf with
+    match scan() with
 	Eof    -> []
       | Ignore -> get_name_list()
       | tok    -> tok :: get_name_list()
@@ -335,20 +336,27 @@ let check_attribute_value_lexically lexerset x t v =
 ;;
 
 
-let split_attribute_value lexerset v =
+let split_attribute_value (lfactory:lexer_factory) v =
   (* splits 'v' into a list of names or nmtokens. The white space separating
    * the names/nmtokens in 'v' is suppressed and not returned.
    *)
-  let lexbuf = Pxp_lexing.from_string_inplace v in
+  (* print_string ("split_attribute_value \"" ^ v ^ "\" = "); *)
+  let lexobj = lfactory # open_string_inplace v in
+  let scan = lexobj # scan_name_string in
   let rec get_name_list() =
-    match lexerset.scan_name_string lexbuf with
+    match scan() with
 	Eof         -> []
       | Ignore      -> get_name_list()
       | Name s      -> s :: get_name_list()
       | Nametoken s -> s :: get_name_list()
       | _           -> raise(Validation_error("Illegal attribute value"))
   in
-  get_name_list()
+  let l = get_name_list() in
+  (* print_string "[";
+     print_string (String.concat "," l);
+     print_string "]\n";
+  *)
+  l
 ;;
 
 
@@ -371,11 +379,12 @@ let rev_concat l =
 ;;
 
 
-let normalize_line_separators lexerset s =
+let normalize_line_separators (lfactory:lexer_factory) s =
   (* Note: Returns [s] if [s] does not contain LFs *)
-  let lexbuf = Pxp_lexing.from_string_inplace s in
+  let lexobj = lfactory # open_string_inplace s in
+  let scan = lexobj # scan_for_crlf in
   let rec get_string l =
-    match lexerset.scan_for_crlf lexbuf with
+    match scan() with
 	Eof        -> l
       | CharData s -> get_string (s::l)
       | _          -> assert false
@@ -387,7 +396,7 @@ let normalize_line_separators lexerset s =
 ;;
 
 
-let value_of_attribute_aux lexerset dtd n atype v =
+let value_of_attribute_aux (lfactory:lexer_factory) dtd n atype v =
   (* See value_of_attribute below. *)
 
   let lexical_error() =
@@ -395,7 +404,7 @@ let value_of_attribute_aux lexerset dtd n atype v =
 
   let remove_leading_and_trailing_spaces u =
     (* Precondition: 'u' matches <name> or <nmtoken> *)
-    match split_attribute_value lexerset u with
+    match split_attribute_value lfactory u with
 	[ u' ] -> u'
       | _      -> assert false
   in
@@ -415,26 +424,26 @@ let value_of_attribute_aux lexerset dtd n atype v =
 	Value v
 
     | (A_id | A_idref | A_nmtoken) ->
-	check_attribute_value_lexically lexerset (lexical_error()) atype v;
+	check_attribute_value_lexically lfactory (lexical_error()) atype v;
 	Value (remove_leading_and_trailing_spaces v)
     | A_entity ->
-	check_attribute_value_lexically lexerset (lexical_error()) atype v;
+	check_attribute_value_lexically lfactory (lexical_error()) atype v;
 	let v' = remove_leading_and_trailing_spaces v in
 	check_ndata_entity v';
 	Value v'
 
     | (A_idrefs | A_nmtokens) ->
-	check_attribute_value_lexically lexerset (lexical_error()) atype v;
-	Valuelist (split_attribute_value lexerset v)
+	check_attribute_value_lexically lfactory (lexical_error()) atype v;
+	Valuelist (split_attribute_value lfactory v)
 
     | A_entities ->
-	check_attribute_value_lexically lexerset (lexical_error()) atype v;
-	let l = split_attribute_value lexerset v in
+	check_attribute_value_lexically lfactory (lexical_error()) atype v;
+	let l = split_attribute_value lfactory v in
 	List.iter check_ndata_entity l;
 	Valuelist l
 
     | A_notation nl ->
-	check_attribute_value_lexically lexerset (lexical_error()) atype v;
+	check_attribute_value_lexically lfactory (lexical_error()) atype v;
 	let v' = remove_leading_and_trailing_spaces v in
 	if not (List.mem v' nl) then
 	  raise(Validation_error
@@ -443,7 +452,7 @@ let value_of_attribute_aux lexerset dtd n atype v =
 	Value v'
 
     | A_enum enuml ->
-	check_attribute_value_lexically lexerset (lexical_error()) atype v;
+	check_attribute_value_lexically lfactory (lexical_error()) atype v;
 	let v' = remove_leading_and_trailing_spaces v in
 	if not (List.mem v' enuml) then
 	  raise(Validation_error
@@ -453,7 +462,7 @@ let value_of_attribute_aux lexerset dtd n atype v =
 ;;
 
 
-let value_of_attribute lexerset dtd n atype v =
+let value_of_attribute (lfactory:lexer_factory) dtd n atype v =
   (* The attribute with name 'n', type 'atype' and string value 'v' is
    * decomposed, and the att_value is returned:
    * - It is checked whether 'v' conforms to the lexical rules for attributes
@@ -485,13 +494,13 @@ let value_of_attribute lexerset dtd n atype v =
     (* The most frequent case *)
     Value v
   else
-    value_of_attribute_aux lexerset dtd n atype v
+    value_of_attribute_aux lfactory dtd n atype v
       (* Note that value_of_attribute_aux allocates memory for the local
        * functions even if they are not called at all.
        *)
 ;;
 
-let check_value_of_attribute lexerset dtd n atype av =
+let check_value_of_attribute (lfactory:lexer_factory) dtd n atype av =
   (* This function checks whether the decomposed attribute value av
    * matches the attribute type, i.e. it checks whether av is the
    * result of the above function value_of_attribute for some
@@ -542,7 +551,7 @@ let check_value_of_attribute lexerset dtd n atype av =
 	(match av with
 	   | Valuelist _ -> unexpected_valuelist()
 	   | Value v ->
-	       check_attribute_value_lexically lexerset (lexical_error()) atype v;
+	       check_attribute_value_lexically lfactory (lexical_error()) atype v;
 	       no_leading_and_trailing_spaces v
 	   | _ -> ()
 	)
@@ -550,7 +559,7 @@ let check_value_of_attribute lexerset dtd n atype av =
 	(match av with
 	   | Valuelist _ -> unexpected_valuelist()
 	   | Value v ->
-	       check_attribute_value_lexically lexerset (lexical_error()) atype v;
+	       check_attribute_value_lexically lfactory (lexical_error()) atype v;
 	       no_leading_and_trailing_spaces v;
 	       check_ndata_entity v
 	   | _ -> ()
@@ -564,7 +573,7 @@ let check_value_of_attribute lexerset dtd n atype av =
 		 if atype = A_idrefs then A_id else A_nmtoken in
 	       List.iter
 		 (check_attribute_value_lexically 
-		    lexerset (lexical_error()) subst_type) 
+		    lfactory (lexical_error()) subst_type) 
 		 l
 	   | _ -> ()
 	)
@@ -575,7 +584,7 @@ let check_value_of_attribute lexerset dtd n atype av =
 	       List.iter no_leading_and_trailing_spaces l;
 	       List.iter
 		 (check_attribute_value_lexically 
-		    lexerset (lexical_error()) A_entity) 
+		    lfactory (lexical_error()) A_entity) 
 		 l;
 	       List.iter check_ndata_entity l
 	   | _ -> ()
@@ -584,7 +593,7 @@ let check_value_of_attribute lexerset dtd n atype av =
 	(match av with
 	   | Valuelist _ -> unexpected_valuelist()
 	   | Value v ->
-	       check_attribute_value_lexically lexerset (lexical_error()) atype v;
+	       check_attribute_value_lexically lfactory (lexical_error()) atype v;
 	       no_leading_and_trailing_spaces v;
 	       if not (List.mem v nl) then
 		 raise(Validation_error
@@ -596,7 +605,7 @@ let check_value_of_attribute lexerset dtd n atype av =
 	(match av with
 	   | Valuelist _ -> unexpected_valuelist()
 	   | Value v ->
-	       check_attribute_value_lexically lexerset (lexical_error()) atype v;
+	       check_attribute_value_lexically lfactory (lexical_error()) atype v;
 	       no_leading_and_trailing_spaces v;
 	       if not (List.mem v enuml) then
 		 raise(Validation_error
@@ -606,7 +615,7 @@ let check_value_of_attribute lexerset dtd n atype av =
 	)
 ;;
 
-let normalization_changes_value lexerset atype v =
+let normalization_changes_value (lfactory:lexer_factory) atype v =
   (* Returns true if:
    * - 'atype' is a "single-value" type, and the normalization of the string
    *   value 'v' of this type discards leading and/or trailing spaces
@@ -635,7 +644,7 @@ let normalization_changes_value lexerset atype v =
 	 * a different string than 'v'.
 	 * This check works for both ISO-8859-1 and UTF-8.
 	 *)
-	let l = split_attribute_value lexerset v in
+	let l = split_attribute_value lfactory v in
 	let v' = String.concat " " l in
 	v <> v'
 ;;
@@ -783,163 +792,3 @@ let write_data_string ~(from_enc:rep_encoding) ~to_enc os content =
 ;;
 
 
-(* ======================================================================
- * History:
- *
- * $Log: pxp_aux.ml,v $
- * Revision 1.18  2003/11/04 22:45:07  gerd
- * 	Problems with Lexing under 3.07 fixed
- *
- * Revision 1.17  2003/06/20 15:14:13  gerd
- * 	Introducing symbolic warnings, expressed as polymorphic
- * variants
- *
- * Revision 1.16  2003/06/15 12:23:21  gerd
- * 	Moving core type definitions to Pxp_core_types
- *
- * Revision 1.15  2002/03/13 22:45:42  gerd
- * 	Improved Pxp_lexing.
- *
- * Revision 1.14  2002/02/20 00:42:23  gerd
- * 	New: rev_concat
- * 	Improved normalize_line_separators; does no longer blow up
- * memory if many lines are found
- *
- * Revision 1.13  2001/10/12 21:38:14  gerd
- * 	Changes for O'caml 3.03-alpha.
- *
- * Revision 1.12  2001/06/27 23:34:54  gerd
- * 	StringMap now here.
- * 	New function extract_prefix.
- *
- * Revision 1.11  2001/06/07 22:51:10  gerd
- * 	New function: check_value_of_attribute (needed for
- * ~att_values option of create_element_node)
- *
- * Revision 1.10  2001/05/17 21:38:53  gerd
- * 	New function namespace_split.
- *
- * Revision 1.9  2001/02/01 20:38:49  gerd
- * 	New support for PUBLIC identifiers.
- *
- * Revision 1.8  2000/10/01 19:45:28  gerd
- * 	New module: Str_hashtbl.
- * 	Many optimizations, especially expand_attvalue and count_lines.
- *
- * Revision 1.7  2000/09/17 00:10:59  gerd
- * 	Optimizations in expand_attvalue
- *
- * Revision 1.6  2000/08/14 22:24:55  gerd
- * 	Moved the module Pxp_encoding to the netstring package under
- * the new name Netconversion.
- *
- * Revision 1.5  2000/07/25 00:30:01  gerd
- * 	Added support for pxp:dtd PI options.
- *
- * Revision 1.4  2000/07/16 18:31:09  gerd
- * 	The exception Illegal_character has been dropped.
- *
- * Revision 1.3  2000/07/16 16:33:57  gerd
- * 	New function write_markup_string: Handles the encoding
- * of the string.
- *
- * Revision 1.2  2000/07/08 22:15:45  gerd
- * 	[Merging 0.2.10:] write_data_string: The character '%' is special, too.
- *
- * Revision 1.1  2000/05/29 23:48:38  gerd
- * 	Changed module names:
- * 		Markup_aux          into Pxp_aux
- * 		Markup_codewriter   into Pxp_codewriter
- * 		Markup_document     into Pxp_document
- * 		Markup_dtd          into Pxp_dtd
- * 		Markup_entity       into Pxp_entity
- * 		Markup_lexer_types  into Pxp_lexer_types
- * 		Markup_reader       into Pxp_reader
- * 		Markup_types        into Pxp_types
- * 		Markup_yacc         into Pxp_yacc
- * See directory "compatibility" for (almost) compatible wrappers emulating
- * Markup_document, Markup_dtd, Markup_reader, Markup_types, and Markup_yacc.
- *
- * ======================================================================
- * Old logs from markup_aux.ml:
- *
- * Revision 1.12  2000/05/27 19:08:30  gerd
- * 	Added functionality to check standalone declaration:
- *
- * 	expand_attvalue: Checks whether included entities violate the
- * stand-alone declaration.
- *
- * 	value_of_attribute: Checks whether ENTITY/ENTITIES values violate
- * this declaration. (Furthermore, it is checked whether the NDATA
- * entity exists - this has been forgotten in previous versions.)
- *
- * 	value_of_attribute/check_attribute_value_lexically: improved.
- *
- * 	New function normalization_changes_value: helps detecting
- * one case which violates the standalone declaration.
- *
- * Revision 1.11  2000/05/20 20:31:40  gerd
- * 	Big change: Added support for various encodings of the
- * internal representation.
- *
- * Revision 1.10  2000/05/01 20:41:56  gerd
- * 	New function write_data_string.
- *
- * Revision 1.9  2000/04/30 18:11:31  gerd
- * 	New function normalize_line_separators.
- * 	In function expand_attvalue: New argument norm_crlf. If the attvalue
- * is read directly from a file, the sequence CR LF must be converted to a
- * single space. If the attvalue is read from a replacement text, CR LF has
- * already converted to a single LF, and CR LF, if still occurring, must be
- * converted to two spaces. The caller can indicate the case by passing
- * true/false as norm_crlf.
- *
- * Revision 1.8  1999/09/01 22:51:07  gerd
- * 	Added functions.
- * 	'character' raises Illegal_character if characters are found that
- * do not match the production Char.
- *
- * Revision 1.7  1999/09/01 16:17:37  gerd
- * 	Added function 'check_name'.
- *
- * Revision 1.6  1999/08/15 20:33:19  gerd
- * 	Added: a function that checks public identifiers. Only certain
- * characters may occur in these identifiers.
- * 	Control characters are rejected by the "character" function.
- * 	Bugfix: recursive entity references are detected in attribute
- * expansion
- *
- * Revision 1.5  1999/08/15 02:18:02  gerd
- * 	That '<' is not allowed in attribute values, is a violation
- * of well-formedness, not of the validity; so WF_error is raised.
- *
- * Revision 1.4  1999/08/15 00:20:37  gerd
- * 	When expanding attribute values, references to parameter
- * entities are now resolved by the method "replacement_text" which
- * has an additional return value, and no longer by "attlist_replacement_text".
- * The new return value indicates whether references to external entities
- * have been resolved (directly or indirectly); this is allowed at some
- * locations but not in attribute values.
- *
- * Revision 1.3  1999/08/14 22:05:53  gerd
- * 	Several functions have now a "warner" as argument which is
- * an object with a "warn" method. This is used to warn about characters
- * that cannot be represented in the Latin 1 alphabet.
- *
- * Revision 1.2  1999/08/10 21:35:06  gerd
- * 	The XML/encoding declaration at the beginning of entities is
- * evaluated. In particular, entities have now a method "xml_declaration"
- * which returns the name/value pairs of such a declaration. The "encoding"
- * setting is interpreted by the entity itself; "version", and "standalone"
- * are interpreted by Markup_yacc.parse_document_entity. Other settings
- * are ignored (this does not conform to the standard; the standard prescribes
- * that "version" MUST be given in the declaration of document; "standalone"
- * and "encoding" CAN be declared; no other settings are allowed).
- * 	TODO: The user should be warned if the standard is not exactly
- * fulfilled. -- The "standalone" property is not checked yet.
- *
- * Revision 1.1  1999/08/10 00:35:50  gerd
- * 	Initial revision.
- *
- *
- *)
