@@ -1,4 +1,4 @@
-(* $Id: pxp_dtd_parser.mli,v 1.1 2003/06/15 18:18:34 gerd Exp $
+(* $Id: pxp_dtd_parser.mli 696 2004-08-20 14:49:57Z gerd $
  * ----------------------------------------------------------------------
  * PXP: The polymorphic XML parser for Objective Caml.
  * Copyright by Gerd Stolpmann. See LICENSE for details.
@@ -7,6 +7,9 @@
 open Pxp_types
 open Pxp_dtd
 
+val create_empty_dtd : config -> dtd
+  (* Create an empty DTD *)
+
 val parse_dtd_entity : config -> source -> dtd
   (* Parse an entity containing a DTD (external subset), and return this DTD. *)
 
@@ -18,12 +21,3 @@ val extract_dtd_from_document_entity : config -> source -> dtd
    *)
 
 
-(* ======================================================================
- * History:
- * 
- * $Log: pxp_dtd_parser.mli,v $
- * Revision 1.1  2003/06/15 18:18:34  gerd
- * 	Initial revision
- *
- * 
- *)
